<?php

namespace Teknisa\FIN\Modules\ContaPagar\Controllers;

use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Environment;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\FIN\Modules\ContaPagar\Services\AjusteTituloService;

/**
 * Class AjusteTituloController
 * @package Teknisa\FIN\Modules\ContaPagar\Controllers
 */
class AjusteTituloController extends TeknisaCrud{

    protected $dataSourceName = 'ajbolpag_tipoajpagar';
    protected $ajusteTituloService;

    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param AjusteTituloService $ajusteTituloService
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, AjusteTituloService $ajusteTituloService)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->ajusteTituloService = $ajusteTituloService;
    }

    /**
     * Salva Ajuste Título a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function salvaAjusteBoleto(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $row = $row->getRow();

            $nrorg       = $row['NRORG'];
            $cdfilial    = $row['CDFILIAL'];
            $nrlanbolpag = $row['NRLANBOLPAG'];
            $idajuctpag  = $row['IDAJUCTPAG'];
            $cdajuctpag  = $row['CDAJUCTPAG'];
            $vrajbolpag  = $row['VRAJBOLPAG'];

            // Salva Titulo a Pagar
            $this->ajusteTituloService->salvaAjusteBoleto(
                $nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag
            );

            // Registra Dados
            $response->addDataSet(new DataSet('salvaAjusteBoleto', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }


    /**
     * Deleta Ajuste Título a Pagar
     * @param Row $row
     * @param Response $response
     */
    public function deletaAjusteBoleto(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $row = $row->getRow();

            $nrorg       = $row['NRORG'];
            $cdfilial    = $row['CDFILIAL'];
            $nrlanbolpag = $row['NRLANBOLPAG'];
            $idajuctpag  = $row['IDAJUCTPAG'];
            $cdajuctpag  = $row['CDAJUCTPAG'];
            $vrajbolpag  = $row['VRAJBOLPAG'];

            // Deleta Titulo a Pagar
            $this->ajusteTituloService->deletaAjusteBoleto(
                $nrorg, $cdfilial, $nrlanbolpag, $idajuctpag, $cdajuctpag, $vrajbolpag
            );

            // Registra Dados
            $response->addDataSet(new DataSet('deletaAjusteBoleto', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}