<?php

namespace Teknisa\FIN\Modules\ContaReceber\Factory;

use Teknisa\Model\Financeiro\Entities\Boletreceber;
use Teknisa\Model\Financeiro\Entities\Baixarecebida;
use Teknisa\Model\Financeiro\Entities\Cnabrec;
use Teknisa\Model\Financeiro\Entities\Errcnabbolr;
use Teknisa\Model\Financeiro\Entities\Emprbanc;
use Teknisa\Model\Financeiro\Entities\Ajbolrec;
use Teknisa\Model\Financeiro\Entities\GeralRec;
use Teknisa\Model\Financeiro\Entities\Contareceber;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;

class RetornoCobrancaFactory {

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    /**
     * Busca Empresa Banco
     * @param $cdbanco
     * @param $cdempresa
     * @param $nrorg
     * @return Emprbanc
     */
    public function buscaEmprbanc($cdbanco, $cdempresa, $nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg']     = $nrorg;
        $params['cdbanco']   = $cdbanco;
        $params['cdempresa'] = $cdempresa;

        /** @var Emprbanc $emprbanc */
        $emprbanc = $this->entityManager->getRepository(Emprbanc::class)->findOneBy($params);

        return $emprbanc;
    }

    /**
     * Busca Boleto Receber
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $nrorg
     * @return Boletreceber
     */
    public function buscaBoletreceber($cdfilial, $nrlanbolrec, $nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['nrlanbolrec'] = $nrlanbolrec;
        $params['cdfilial']    = $cdfilial;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findOneBy($params);

        return $boletreceber;
    }

    /**
     * Busca Boleto Receber Retorno
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $nrcnabrecret
     * @return Boletreceber
     */
    public function buscaBoletreceberRetorno($nrorg, $cdfilial, $nrlanbolrec, $nrcnabrecret){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanbolrec']  = $nrlanbolrec;
        $params['nrcnabrecret'] = $nrcnabrecret;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findOneBy($params);

        return $boletreceber;
    }

    /**
     * Busca CNAB Receber
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabrec
     * @return Cnabrec
     */
    public function buscaCnabrec($nrorg, $cdempresa, $nrcnabrec){
        /** @var $params */
        $params = array();
        $params['nrorg']     = $nrorg;
        $params['cdempresa'] = $cdempresa;
        $params['nrcnabrec'] = $nrcnabrec;

        /** @var Cnabrec $cnabrec */
        $cnabrec = $this->entityManager->getRepository(Cnabrec::class)->findOneBy($params);

        return $cnabrec;
    }

    /**
     * Busca Erros Cnab Boleto Receber
     * @param $nrlanbolrec
     * @param $cdfilial
     * @param $nrorg
     * @return array
     */
    public function buscaErrcnabbolrs($nrlanbolrec, $cdfilial, $nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['nrlanbolrec'] = $nrlanbolrec;
        $params['cdfilial']    = $cdfilial;

        /** @var Errcnabbolr $errcnabbolps */
        $errcnabbolr = $this->entityManager->getRepository(Errcnabbolr::class)->findBy($params);

        return $errcnabbolr;
    }

    /**
     * Baixa Recebida
     * @param $cdfilial
     * @param $nrbaixarec
     * @param $nrorg
     * @return Baixarecebida
     */
    public function buscaBaixarecebida($cdfilial, $nrbaixarec, $nrorg){
        /** @var  $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['nrbaixarec'] = $nrbaixarec;
        $params['cdfilial']   = $cdfilial;

        /** @var Baixarecebida $baixarecebida */
        $baixarecebida = $this->entityManager->getRepository(Baixarecebida::class)->findOneBy($params);

        return $baixarecebida;
    }
    
    /**
     * Busca Contas Receber
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @return Contareceber
     */
    public function buscaContasreceber($nrorg, $cdfilibolrec, $nrlanbolrec){
    	/** @var  $params */
    	$params = array();
    	$params['nrorg']        = $nrorg;
    	$params['cdfilibolrec'] = $cdfilibolrec;
    	$params['nrlanbolrec']  = $nrlanbolrec;
    
    	/** @var Contareceber $contareceber */
    	$contareceber = $this->entityManager->getRepository(Contareceber::class)->findBy($params);
    
    	return $contareceber;
    }

    /**
     * Insere Cnab Receber
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabrec
     * @param $idtipcnabrec
     * @param $cdoperador
     * @param $cdbanco
     * @param $dsArquivo
     * @return Cnabrec
     */
    public function insereCnabrec(
        $nrorg, $cdempresa, $nrcnabrec, $idtipcnabrec, $cdoperador, $cdbanco, $dsArquivo
    ){
        $cnabrec = new Cnabrec();
        $cnabrec->setNrorg($nrorg);
        $cnabrec->setCdempresa($cdempresa);
        $cnabrec->setNrcnabrec($nrcnabrec);
        $cnabrec->setDsarqcnabrec($dsArquivo);
        $cnabrec->setIdtipcnabrec($idtipcnabrec);
        $cnabrec->setDtcnabrec(DateUtil::getDataAtual());
        $cnabrec->setCdoperador($cdoperador);
        $cnabrec->setCdbanco($cdbanco);
        $cnabrec->setDtinclusao(DateUtil::getDataAtual());

        return $cnabrec;
    }

    /**
     * Insere Baixa Recebida
     * @param $cdempresa
     * @param $cdfilial
     * @param $cdtpbaixarec
     * @param $dtbaixarec
     * @param $cdreducont
     * @param $cdoperador
     * @param $nrorg
     * @param $nrbaixarec
     * @param $vrbaixa
     * @param $nrcnabrec
     * @param $idstatusrec
     * @return Baixarecebida
     */
    public function insereBaixarecebida(
        $cdempresa, $cdfilial, $cdtpbaixarec, $dtbaixarec, $cdreducont, $cdoperador,
        $nrorg, $nrbaixarec, $vrbaixa, $nrcnabrec, $idstatusrec
    ){
        $baixarecebida = new Baixarecebida();
        $baixarecebida->setNrorg($nrorg);
        $baixarecebida->setCdempresa($cdempresa);
        $baixarecebida->setCdfilial($cdfilial);
        $baixarecebida->setNrbaixarec($nrbaixarec);
        $baixarecebida->setDtbaixarec($dtbaixarec);
        $baixarecebida->setVrbaixarec($vrbaixa);
        $baixarecebida->setIdstatusbrec($idstatusrec);
        $baixarecebida->setCdtpbaixarec($cdtpbaixarec);
        $baixarecebida->setCdreducont($cdreducont);
        $baixarecebida->setCdoperador($cdoperador);
        $baixarecebida->setDsbaixarec("Baixa realizada via importação CNAB. Retorno nº {$nrcnabrec}");
        $baixarecebida->setDtlancbaixar(DateUtil::getDataAtual());
        $baixarecebida->setDtinclusao(DateUtil::getDataAtual());
        $baixarecebida->setDtultatu(DateUtil::getDataAtual());
        $baixarecebida->setCdoperinclusao($cdoperador);
        $baixarecebida->setCdoperultatu($cdoperador);

        return $baixarecebida;
    }

    /**
     * Insere Erros Cnab Boleto Receber
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdbanco
     * @param $cdercnabbolr
     * @param $nrorg
     * @return Errcnabbolr
     */
    public function insereErrcnabbolr($cdfilial, $nrlanbolrec, $cdbanco, $cdercnabbolr, $nrorg){
        $errcnabbolp = new Errcnabbolr();
        $errcnabbolp->setNrorg($nrorg);
        $errcnabbolp->setCdfilial($cdfilial);
        $errcnabbolp->setNrlanbolrec($nrlanbolrec);
        $errcnabbolp->setCdercnabbolr($cdercnabbolr);
        $errcnabbolp->setCdbanco($cdbanco);
        $errcnabbolp->setDtinclusao(DateUtil::getDataAtual());

        return $errcnabbolp;
    }

    /**
     * Atualiza Boleto Receber
     * @param Boletreceber $boletreceber
     * @param $vrfinalbrec
     * @param $cdfilibairec
     * @param $nrbaixarec
     * @param $nrcnabrecret
     * @param $idsitucnabr
     * @param $nrorg
     * @param $idstatbrec
     * @return Boletreceber
     */
    public function atualizaBoletoreceber(
        Boletreceber $boletreceber, $vrfinalbrec, $cdfilibairec, $nrbaixarec, $nrcnabrecret, $idsitucnabr, $nrorg, $idstatbrec
    ){
        $boletreceber->setVrfinalbrec($vrfinalbrec);
        $boletreceber->setCdfilibairec($cdfilibairec);
        $boletreceber->setNrbaixarec($nrbaixarec);
        $boletreceber->setNrcnabrecret($nrcnabrecret);
        $boletreceber->setIdsitucnabr($idsitucnabr);
        $boletreceber->setIdstatbrec($idstatbrec);
        $boletreceber->setNrorgultatu($nrorg);
        $boletreceber->setDtultatu(DateUtil::getDataAtual());

        return $boletreceber;
    }
    
    /**
     * Atualiza Conta Receber
     * @param Contareceber $contareceber
     * @param $idstatrec
     * @return Contareceber
     */
    public function atualizaContareceber(
    	Contareceber $contareceber, $idstatrec
    ){
    	$contareceber->setIdstatrec($idstatrec);
    	
    	return $contareceber;
    }

    public function atualizaCarteiraBoletreceber($boletreceber, $dtocorrencia, $vrtaxacartorio){
        $boletreceber->setVrtaxacartorio($vrtaxacartorio);
        $boletreceber->setDtcobtaxcart($dtocorrencia);

        return $boletreceber;
    }

    public function associaBoletoreceberCnab(
        Boletreceber $boletreceber, $cdcartcobbol, $idcartcobrec, $nrcnabrecret, $idsitucnabr, $dtiniprotrec, $idprotestrec, $nrorg
    ){
        /*
         * Busca a posição 54 do segmento T
         * Foi necessario comentar a atualização do CDCARTCOBBOL, passado por parametro,
         * porque não condiz com o numero da carteira mesmo e sim com o identificador
         */
        //$boletreceber->setCdcartcobbol($cdcartcobbol);
        //$boletreceber->setIdcartcobrec($idcartcobrec);
        $boletreceber->setNrcnabrecret($nrcnabrecret);
        $boletreceber->setIdsitucnabr($idsitucnabr);
        $boletreceber->setDtiniprotrec($dtiniprotrec);
        $boletreceber->setIdprotestrec($idprotestrec);
        $boletreceber->setNrorgultatu($nrorg);
        $boletreceber->setDtultatu(DateUtil::getDataAtual());

        return $boletreceber;
    }

    /**
     * Atualiza Valor Baixa Recebida
     * @param Baixarecebida $baixarecebida
     * @param $vrbaixarec
     * @return Baixarecebida
     */
    public function atualizaValorBaixarecebida(Baixarecebida $baixarecebida, $vrbaixarec){
        $baixarecebida->setVrbaixarec($baixarecebida->getVrbaixarec() + $vrbaixarec);

        return $baixarecebida;
    }

    /**
     * Busca Geral Rec
     * @param $nrorg
     * @return GeralRec
     */
    public function buscaGeralRec($nrorg){
        /** @var  $params */
        $params = array();
        $params['nrorg']      = $nrorg;

        /** @var GeralRec $geralrec */
        $geralrec = $this->entityManager->getRepository(GeralRec::class)->findOneBy($params);

        return $geralrec;
    }

    /**
     * Insere Ajuste Título
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdajuctrec
     * @param $vrajbolrec
     * @param $idorigajbrec
     * @param $idtpajbolrec
     * @return Ajbolrec
     */
    public function inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $vrajbolrec, $idorigajbrec, $idtpajbolrec){
        /** @var Ajbolrec $ajuste */
        $ajuste = new Ajbolrec();
        
        $ajuste->setNrorg($nrorg);
        $ajuste->setCdfilial($cdfilial);
        $ajuste->setNrlanbolrec($nrlanbolrec);
        $ajuste->setCdajuctrec($cdajuctrec);
        $ajuste->setVrajbolrec($vrajbolrec);
        $ajuste->setIdorigajbrec($idorigajbrec);
        $ajuste->setIdtpajbolrec($idtpajbolrec);
        $ajuste->setDtinclusao(DateUtil::getDataAtual());
        
        return $ajuste;
    }
}