<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\Bco00500RecalculoOrcamentoFluxoCaixaFactory;

class Bco00500RecalculoOrcamentoFluxoCaixaService{

    protected $dataSourceName, $masks, $service;

    public function __construct($entityManager, $environment, Bco00500RecalculoOrcamentoFluxoCaixaFactory $recalculoOrcamentoFluxoCaixaFactory) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->recalculoOrcamentoFluxoCaixaFactory = $recalculoOrcamentoFluxoCaixaFactory;
        $this->nrorg = $this->environment->getNrOrgTrab();
    }

    /**
     * [saveOrcamentoFluxoCaixa Realiza o salvamento das cadastramento automático
     * para todas as classificações financeiras por empresa informada.]
     */
    public function saveOrcamentoFluxoCaixa($orcamentos, $row) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            // Busca Parâmetros Gerais
            $paramGeral = $this->recalculoOrcamentoFluxoCaixaFactory->getParamGeral($this->nrorg);
            
            // Verifica Tipo de Orçamento Financeiro
            if ($paramGeral->getIdtporcafina() == 'F')
                 $filial = $row['CDFILIAL'];
            else $filial = $this->environment->getCdfilial();
            
            // Percorre Orçamentos Financeiros
            foreach ($orcamentos as $orcamento) {
                
                // Formata Data de Orçamento Financeiro
                $dataMes = '01/'.$orcamento['DTMES'].'/'.$row['DTMES'];
                $dataMes = $this->formataData($dataMes);
                
                // Busca Orçamento Financeiro
                $orcafina = $this->recalculoOrcamentoFluxoCaixaFactory->getOrcafina(
                    $row['CDEMPRESA'], $filial, $row['CDCLASSFINA'], $dataMes, $this->nrorg
                );
                
                // Verifica existência Orçamento Financeiro
                if (!$orcafina){
                    // Insere Orçamento Financeiro
                    $orcafina = $this->recalculoOrcamentoFluxoCaixaFactory->insereOrcafina(
                        0, $orcamento['VREXTOLEORCA'], $orcamento['VRORCAFINA'], $row['CDEMPRESA'],
                        $filial, $row['CDCLASSFINA'], $dataMes, $this->nrorg
                    );
                }else{
                    // Atualiza Orçamento Financeiro
                    $orcafina->setVrextoleorca($orcamento['VREXTOLEORCA']);
                    $orcafina->setVrorcafina($orcamento['VRORCAFINA']);
                }
                
                $this->entityManager->persist($orcafina);
                $this->entityManager->flush();
            }

            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * [CadastramentoAutomaticoDoOrcamento Realiza o cadastramento automático
     * para todas as classificações financeiras por empresa informada.]
     */
    public function CadastramentoAutomaticoDoOrcamento($row, $origem, $destino) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $paramGeral = $this->recalculoOrcamentoFluxoCaixaFactory->getParamGeral($this->nrorg);
            $anoInicial = '01/01/'.$row['DTMES'];
            $anoFinal = '31/12/'.$row['DTMES'];

            $tipoCadastramento = null;
            
            if ($paramGeral->getIdtporcafina() == 'F'){
                $tipoCadastramento = 'F';
                $orcamentoOrigem = $this->recalculoOrcamentoFluxoCaixaFactory->getPeriodoOrcaFina(
                    $origem['CDEMPRESA'],
                    $origem['CDFILIAL'],
                    $anoInicial,
                    $anoFinal
                );

            }else{
                $filialOrigem = $this->environment->getCdfilial();
                $tipoCadastramento = 'E';
                $orcamentoOrigem = $this->recalculoOrcamentoFluxoCaixaFactory->getPeriodoOrcaFina(
                    $origem['CDEMPRESA'],
                    $filialOrigem,
                    $anoInicial,
                    $anoFinal
                );
            }
            
            foreach ($orcamentoOrigem as $origem) {
                if ($tipoCadastramento == 'E'){
                    $this->efetuaOrcamentoPorEmpresa(
                        $filialOrigem,
                        $origem,
                        $destino,
                        $paramGeral,
                        $row['OPCAOOPERACAO']
                    );
                }else{
                    $this->efetuaOrcamentoPorFilial(
                        $origem,
                        $destino,
                        $row['OPCAOOPERACAO']
                    );
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * [efetuaOrcamentoPorFilial Realiza o cadastramento Automático
     * da filial de origem para N filiais de destino]
     *
     * @param  [type] $origem  [Filial do orçamento a ser replicado.
     * Caso a filial não esteja parametrizada na tela orçamento para
     * financeiro, a filial considerada será a filial logada]
     * @param  [type] $destino [Filiais que receberão a replicação]
     * @param  [type] $opcao   [description]
     */
    public function efetuaOrcamentoPorFilial($origem, $destino, $opcao){
        // Unidades para Inclusão/Sobreposição
        $filiais = $destino['CDFILIAL'];
        
        // Percorre Unidades p/ Inclusão/Sobreposição
        foreach ($filiais as $filial) {
            
            // Busca Orçamento Financeiro de Destino
            $orcafinaDestino = $this->recalculoOrcamentoFluxoCaixaFactory->getOrcafina(
                $destino['CDEMPRESA'], $filial, $origem['CDCLASSFINA'],
                $this->formataData($origem['DTORCAFINA']), $this->nrorg
            );
            
            // Verifica Inclusão/Sobreposição
            if ($opcao == 'I'){
                if (!$orcafinaDestino)
                    $this->insereOrcafina($origem, $filial, $destino);
            }else{
                if ($orcafinaDestino){
                    if (($orcafinaDestino->getCdempresa() != $origem['CDEMPRESA']) || ($orcafinaDestino->getCdfilial() != $origem['CDFILIAL'])){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) > $orcafinaDestino->getVrrealorca()){
                            if ($orcafinaDestino)
                                 $this->updateOrcafina($orcafinaDestino, $origem);
                            else $this->insereOrcafina($origem, $filial, $destino);
                        }
                    }
                }else $this->insereOrcafina($origem, $filial, $destino);
            }
        }
    }

    /**
     * [efetuaOrcamentoPorEmpresa Realiza o cadastramento Automático
     * da empresa de origem para a empresa de destino]
     */
    public function efetuaOrcamentoPorEmpresa($filial, $origem, $destino, $paramGeral, $opcao){
        $orcafinaDestino = $this->recalculoOrcamentoFluxoCaixaFactory->getOrcafina(
            $destino['CDEMPRESA'],
            $filial,
            $origem['CDCLASSFINA'],
            $this->formataData($origem['DTORCAFINA']),
            $this->nrorg
        );
        if ($opcao == 'I'){
            if ($orcafinaDestino){
                if (($orcafinaDestino->getVrextoleorca() == 0) && ($orcafinaDestino->getVrorcafina() == 0)){
                    if ($paramGeral->getIdconsorcfin() == 'S'){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) >= $orcafinaDestino->getVrrealorca()){
                            $this->updateOrcafina($orcafinaDestino, $origem);
                        }
                    }else{
                        $this->updateOrcafina($orcafinaDestino, $origem);
                    }
                }
            }else{
                $this->insereOrcafina($origem, $filial, $destino);
            }
        }else{
            if ($orcafinaDestino){
                if (($orcafinaDestino->getCdempresa() != $origem['CDEMPRESA']) ||
                    ($orcafinaDestino->getCdfilial() != $origem['CDFILIAL'])){
                    if ($paramGeral->getIdconsorcfin() == 'S'){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) >= $orcafinaDestino->getVrrealorca()){
                            $this->updateOrcafina($orcafinaDestino, $origem);
                        }
                    }else{
                        $this->updateOrcafina($orcafinaDestino, $origem);
                    }
                }
            }else{
                $this->insereOrcafina($origem, $filial, $destino);
            }
        }
    }

    public function insereOrcafina($origem, $cdfilial, $destino){
        $orcafinaDestino = $this->recalculoOrcamentoFluxoCaixaFactory->insereOrcafina(
            0,
            $origem['VREXTOLEORCA'],
            $origem['VRORCAFINA'],
            $destino['CDEMPRESA'],
            $cdfilial,
            $origem['CDCLASSFINA'],
            $this->formataData($origem['DTORCAFINA']),
            $this->nrorg
        );
        $this->entityManager->persist($orcafinaDestino);
        $this->entityManager->flush();
    }

    public function updateOrcafina($orcafinaDestino, $origem){
        $orcafinaDestino->setVrextoleorca($origem['VREXTOLEORCA']);
        $orcafinaDestino->setVrorcafina($origem['VRORCAFINA']);
        $this->entityManager->persist($orcafinaDestino);
        $this->entityManager->flush();
    }

    public function recalcularOrcamentoFluxoCaixa($classificacoesFinanceiras, $row) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $paramGeral = $this->recalculoOrcamentoFluxoCaixaFactory->getParamGeral($this->nrorg);
            foreach ($classificacoesFinanceiras as $classificacao) {
                $periodos = $this->recalculoOrcamentoFluxoCaixaFactory->getDatasPeriodos(
                    $row['DTORCAFINA_INI'],
                    $row['DTORCAFINA_FIN'],
                    $this->nrorg
                );

                foreach($periodos as $periodo){
                    $valorTotal = 0;
                    list($dia, $mes, $ano) = explode("/", $periodo['DTMES']);
                    $ultimoDia = cal_days_in_month(CAL_GREGORIAN, $mes , $ano);
                    $ultimoDiaMes = $ultimoDia.'/'.$mes.'/'.$ano;
                    $tipoRecalculo = null;
                    $filial = null;
                    
                    if ($paramGeral->getIdtporcafina() == 'F'){
                        $filial = $row['CDFILIAL'];
                        $tipoRecalculo = 'F';
                    }else{
                        $filial = $this->environment->getCdfilial();
                        $tipoRecalculo = 'E';
                    }
                    
                    $valorTotal = $this->getValorTotalTitulos(
                        $row['CDEMPRESA'], $classificacao['CDCLASSFINA'],
                        $filial, $periodo['DTMES'], $ultimoDiaMes
                    );
                    //var_dump($row['CDEMPRESA'].' - '.$filial.' - '.$classificacao['CDCLASSFINA'].' - '.$this->formataData($periodo['DTMES']).' - '.$this->nrorg);
                    $orcafina = $this->recalculoOrcamentoFluxoCaixaFactory->getOrcafina(
                        $row['CDEMPRESA'], $filial, $classificacao['CDCLASSFINA'],
                        $this->formataData($periodo['DTMES']), $this->nrorg
                    );
                    if ($orcafina){
                        $orcafina->setVrrealorca($valorTotal);
                        $this->entityManager->persist($orcafina);
                        $this->entityManager->flush();
                    }else{
                        if ($valorTotal != 0){
                            $orcafina = $this->recalculoOrcamentoFluxoCaixaFactory->insereOrcafina(
                                $valorTotal, 0, 0, $row['CDEMPRESA'], $filial,
                                $classificacao['CDCLASSFINA'], $this->formataData($periodo['DTMES']), $this->nrorg
                            );
                            $this->entityManager->persist($orcafina);
                            $this->entityManager->flush();
                        }
                    }
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    public function getValorTotalTitulos($cdempresa, $cdclassfina, $cdfilial, $dtmes, $ultimoDiaMes){
        $paramsFilial = array(
            'CDEMPRESA'     => $cdempresa,
            'CDCLASSFINA'   => $cdclassfina,
            'NRORG'         => $this->nrorg,
            'CDFILIAL'      => $cdfilial,
            'DTMOVFLUXO_INI' => $dtmes,
            'DTMOVFLUXO_FIN' => $ultimoDiaMes
        );
        $totalTitulo = 0;
        $totalValorAtual = $this->recalculoOrcamentoFluxoCaixaFactory->getValorTotalContas($paramsFilial);
        $ajustesBoleto = $this->recalculoOrcamentoFluxoCaixaFactory->getTotalAjusteBoleto($paramsFilial);
        $totalTitulo = $totalValorAtual['VRATUAPAG'];
        foreach($ajustesBoleto as $ajuste){
            if ($ajuste['VRAJUSTE']){
                if ($ajuste['TIPOAJUS'] == 'DESCONTO'){
                    $totalTitulo = $totalTitulo - $ajuste['VRAJUSTE'];
                }
                if ($ajuste['TIPOAJUS'] == 'ACRESCIMO'){
                    $totalTitulo = $totalTitulo + $ajuste['VRAJUSTE'];
                }
            }
        }
        return $totalTitulo;
    }

    public function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}
